/**
   This program shows that, while the two implementations of IntBag
   have the same interface, they do not act the same from the point
   of view of performance. Each implementation is faster than the
   other for some operations (e.g., add() and countOccurrences())
   and the two implementations do not use the same amount of memory.
*/
import java.util.Random;
// http://stackoverflow.com/questions/363681/generating-random-integers-in-a-range-with-java

public class BagPerformanceDemo
{
   private static final int ITERATIONS = 100_000_000;

   private static final int RANDOM_RANGE_1 = 1_000;
   private static final int RANDOM_RANGE_2 = ITERATIONS;
   // choose either RANDOM_RANGE_1 or RANDOM_RANGE_2
   private static final int RANDOM_RANGE = RANDOM_RANGE_1;

   public static void main(String[] args)
   {
      Random rand = new Random();

      IntBag bag_1 = new IntArrayBag();
      IntBag bag_2 = new IntArraysBag();
      // choose either bag_1 or bag_2
      IntBag bag = bag_1;

      long startTime = System.nanoTime();
      for (int i = 0; i < ITERATIONS; i++)
      {
         bag.add( rand.nextInt(RANDOM_RANGE) );
      }
      long endTime = System.nanoTime();
      long duration = (endTime - startTime); //divide by 1_000_000 to get milliseconds.

      System.out.println("Filling the bag took " + duration/1_000_000 + " millseconds");

      int[] counts = new int[RANDOM_RANGE];

      startTime = System.nanoTime();
      for (int i = 0; i < RANDOM_RANGE; i++)
      {
         counts[i] = bag.countOccurrences( i );
      }
      endTime = System.nanoTime();
      duration = (endTime - startTime); //divide by 1_000_000 to get milliseconds.

      System.out.println("Counting the bag took " + duration/1_000_000 + " millseconds");

      double avgCount = 0;
      for (int i = 0; i < RANDOM_RANGE; i++)
      {
         avgCount += counts[i];
      }
      avgCount = avgCount / RANDOM_RANGE;
      System.out.println("Each number occurrs on average " + avgCount + " times");
   }
}