/******************************************************************************
* An IntBag is a collection of int numbers.
* The same number may appear multiple times in a bag.
******************************************************************************/

public abstract class IntBag
{
   /**
   * Add a new element to this bag.
   * @param element
   *   the new element that is being inserted
   * @postcondition
   *   A new copy of the element has been added to this bag.
   **/
   public abstract void add(int element);


   /**
   * Add new elements to this bag.
   * @param elements
   *   (a variable-arity argument)
   *   one or more new elements that are being inserted
   * @postcondition
   *   A new copy of the element has been added to this bag.
   **/
   public abstract void addMany(int... elements);


   /**
   * Add the contents of another bag to this bag.
   * @param addend
   *   a bag whose contents will be added to this bag
   * @precondition
   *   The parameter, addend, is not null.
   * @postcondition
   *   The elements from addend have been added to this bag.
   * @exception NullPointerException
   *   Indicates that addend is null.
   **/
// public abstract void addAll(IntBag addend);


   /**
   * Accessor method to count the number of occurrences of a particular element
   * in this bag.
   * @param target
   *   the element that needs to be counted
   * @return
   *   the number of times that target occurs in this bag
   **/
   public abstract int countOccurrences(int target);


   /**
   * Remove one copy of a specified element from this bag.
   * @param target
   *   the element to remove from the bag
   * @postcondition
   *   If target was found in the bag, then one copy of
   *   target has been removed and the method returns true.
   *   Otherwise the bag remains unchanged and the method returns false.
   **/
   public abstract boolean remove(int target);


   /**
   * Determine the number of elements in this bag.
   * @param - none
   * @return
   *   the number of elements in this bag
   **/
   public abstract int size( );


   /**
   * Create a new bag that contains all the elements from two other bags.
   * @param b1
   *   the first of two bags
   * @param b2
   *   the second of two bags
   * @precondition
   *   Neither b1 nor b2 is null.
   * @return
   *   the union of b1 and b2
   * @exception NullPointerException.
   *   Indicates that one of the arguments is null.
   **/
   public static IntBag union(IntBag b1, IntBag b2)
   {
      // Java doesn't allow abstract static methods.
      // So we define a stub method and just hope
      // that each subclass overrides it.
      return null;
   }

}
